#!/usr/bin/env perl

use strict;
use warnings;
use File::Fetch;

# рабочая директория
my $work_dir = '/root/bind/tmp';
# путь, где хранятся файлы зон
my $szc_dir = '/etc/namedb/slave';
# путь, куда будет писаться конфиг для зоны
my $bind_incdir = '/etc/namedb/includes';
# URL, откуда забирать список
my $uri_zf = 'http://www.example.org/named.txt';

# IP адреса авторитативных DNS серверов
my $master_srv = 'ip;';
# шаблон зоны
my $zone_template = 'zone "{zname}" {
	type slave;
	masters { {msrv} };
	file "{fname}";
};';


my $write_conf = 0;
my @zones = ();
my @deleted_zones = ();

# Качаем список доменов
my $ff = File::Fetch->new(uri => $uri_zf);
my $zl_file = $ff->fetch(to => $work_dir) || die 'error: ' . $ff->error;

open(FIN, '<', $zl_file) || die "error: can't open file $zl_file: $!\n";
while (<FIN>){
	chomp; $_ = lc;
	next unless /([a-z1-9\-]+?\.)+?[a-z1-9\-]+/;
	push(@zones, $_);
}
close(FIN);

unlink($zl_file);

# Определяем - были ли добавлены/удалены домены
if (open(FIN, '<', "$work_dir/dns_sync.txt")){
	my @old_zones = ();

	while (<FIN>){ chomp; push(@old_zones, $_); }
	close(FIN);

	$write_conf = ($#zones != $#old_zones);
	foreach (@old_zones){
		if (!($_ ~~ @zones)){
				$write_conf = 1;
				push(@deleted_zones, "$bind_incdir/$_.conf");
				push(@deleted_zones, "$szc_dir/$_.db");
		}
	}
} else {
	$write_conf = 1;
}

# Если были добавлены/удалены домены, то пишем
# конфигурационные файлы, удаляем не используемые
# файлы зон и конфиги к ним
if ($write_conf){
	my $buf = '';
	my @error_zones = ();

	open(FOUT_TMP, '>', "$work_dir/dns_sync.txt") || die "error: can't open file $work_dir/dns_sync.txt: $!\n";
	foreach (@zones){
		if (open(FOUT, '>', "$bind_incdir/$_.conf")){
			$buf = $zone_template;
			$buf =~ s/{zname}/$_/g;
			$buf =~ s/{msrv}/$master_srv/g;
			$buf =~ s/{fname}/$szc_dir\/$_.db/g;
			print(FOUT "$buf\n");
			close(FOUT);

			print(FOUT_TMP "$_\n");
		} else {
			push(@error_zones, "Can't open file $_.conf: $!\n");
		}
	}
	close(FOUT_TMP);

	unlink(@deleted_zones) if ($#deleted_zones >= 0);
	print(@error_zones) if ($#error_zones >= 0);
}

exit(0);

