#!/usr/bin/env perl

use strict;
use warnings;
use File::Fetch;

# рабочая директория
my $work_dir = '/tmp';
# путь, где хранятся файлы зон
my $szc_dir = '/etc/namedb/slave';
# URL, откуда забирать список
my $uri_zf = 'http://www.example.org/test.txt';

# имя конфигурационного файла
my $config_file = 'dns.conf';
# IP адреса авторитативных DNS серверов
my $master_srv = 'ip; ip;';
# шаблон зоны
my $zone_template = 'zone "{zname}" { type slave; masters { {msrv} }; file "{fname}"; };';
# команда управления DNS сервером
my $rndc_bin = '/usr/sbin/rndc';

my $write_conf = 0;
my @zones = ();
my @deleted_zones = ();

# Качаем список доменов
my $ff = File::Fetch->new(uri => $uri_zf);
my $zl_file = $ff->fetch(to => $work_dir) || die 'error: ' . $ff->error;

open(FIN, '<', $zl_file) || die "error: can't open file $zl_file: $!\n";
while (<FIN>){
	chomp; $_ = lc;
	next unless /([a-z1-9\-]+?\.)+?[a-z1-9\-]+/;
	push(@zones, $_);
}
close(FIN);

unlink($zl_file);

# Определяем - были ли добавлены/удалены домены
if (open(FIN, '<', $config_file)){
	my @old_zones = ();

	while (<FIN>){
		chomp; $_ = lc;
		if (/zone[ ]+"(([a-z1-9\-]+?\.)+?[a-z1-9\-]+)"/){
			push(@old_zones, $1);
		}
	}

	$write_conf = ($#zones != $#old_zones);
	foreach (@old_zones){
		if (!($_ ~~ @zones)){
				$write_conf = 1;
				push(@deleted_zones, "$szc_dir/$_.db");
		}
	}

	close(FIN);
} else {
	$write_conf = 1;
}

# Если были добавлены/удалены домены, то пишем
# конфигурационный файл, заставляем сервер перечитать
# конфиги и удаляем не используемые файлы зон
if ($write_conf){
	my $buf = '';
	my $rndc_args = "$rndc_bin reload > /dev/null 2>&1";

	open(FOUT, '>', $config_file) || die "error: can't open file $config_file: $!\n";
	foreach (@zones){
		$buf = $zone_template;
		$buf =~ s/{zname}/$_/g;
		$buf =~ s/{msrv}/$master_srv/g;
		$buf =~ s/{fname}/$szc_dir\/$_.db/g;
		print(FOUT "$buf\n");
	}
	close(FOUT);

	unlink(@deleted_zones) if ($#deleted_zones >= 0);
	system($rndc_args) == 0 || die("command - $rndc_args failed: $?\n");
}

exit(0);
